Option Explicit On 
'---------------------------------------------------------------
 ' Module Name: "SoftTimer"
 '
 ' This creates and manages software timer objects
 '
 ' References: See RR model XXYYZZ.MDL
 '
 ' Created By:
 '
 ' $Workfile: $
 ' $Revision: $
 ' $Date: $
 ' $Log: $
 '---------------------------------------------------------------
Class CSoftTimer

    Private lStart As Long      ' System Time at "Clock Start"
    Private lElapsed As Long    ' Cumulative Time Elapsed
    Private ModuleName As String
    Private bTime As Boolean    ' Indicates Timing in Progress

    ' This sets a "one shot" timer, which disables
    ' itself at the end of the specified interval,
    ' which is specified in seconds.
    Public Sub Wait(ByVal sInterval As Single)
        Dim L As Long
        Dim lTarget As Long

        Const ProcName = "Wait"
        On Error GoTo ErrorHandler

        If sInterval < 0.01 Then Exit Sub ' Ignore delay this short
        lTarget = Environment.TickCount + (1000 * sInterval)
        ' Calculate end time
        While Environment.TickCount < lTarget
            'Yeild()                      ' Wait & Yeild while waiting
        End While
ErrorHandler:
        Select Case Err.Number
            Case 0 'Do nothing
            Case Else : Call ClassErrorHandler(ModuleName, ProcName)
        End Select
    End Sub
    Public Property Elapsed() As Long
        Get
            ' Return the Current elapsed time in ms.
            ' If running, include the current time since start,
            ' else just the elapsed time when stopped
            If bTime Then
                Elapsed = lElapsed + Environment.TickCount - lStart
            Else
                Elapsed = lElapsed
            End If

        End Get
        Set(ByVal Value As Long)

        End Set
    End Property
    ' Reset and Stop the Clock
    Public Sub Reset()
        Const ProcName = "Reset"

        On Error GoTo ErrorHandler

        On Error Resume Next
        lElapsed = 0            ' Set Elapsed Time to Zero
        bTime = False           ' Not Running

ErrorHandler:
        Select Case Err.Number
            Case 0 'Do nothing
            Case Else : Call ClassErrorHandler(ModuleName, ProcName)
        End Select
    End Sub

    ' Start the Clock (If Not Already Running)
    ' Optionally reset the elapsed time counter at the same time
    Public Sub ClockStart(Optional ByVal bReset As Object = False)
        Const ProcName = "ClockStart"
        On Error GoTo ErrorHandler

        On Error Resume Next
        If Not bTime Then lStart = Environment.TickCount
        ' Set Start Time
        bTime = True            ' Disable Restart Until Used
        If Not bReset Is Nothing Then
            If bReset Then lElapsed = 0
        End If              ' Reset elapsed time counter

ErrorHandler:
        Select Case Err.Number
            Case 0 'Do nothing
            Case Else : Call ClassErrorHandler(ModuleName, ProcName)
        End Select
    End Sub
    ' Stop the Clock and add cumulative time to elapsed time counter
    Public Sub ClockStop()
        Const ProcName = "ClockStop"
        On Error GoTo ErrorHandler

        lElapsed = lElapsed + Environment.TickCount - lStart
        ' Set Elapsed Time
        bTime = False           ' Not Running

ErrorHandler:
        Select Case Err.Number
            Case 0 'Do nothing
            Case Else : Call ClassErrorHandler(ModuleName, ProcName)
        End Select
    End Sub
    Public Sub New()
        ModuleName = TypeName(Me)
    End Sub
End Class
