﻿Public Class ucErrorLog

    Private oLogItem As New ListViewItem

    Public Event ErrorLogged(oDetail As ErrorDetail)

    Public Property CollapseRepeatingMessages As Boolean = True
    Public Property ShowHideButton As Boolean = True

    Public Property ShowStatusBar As Boolean = False

    Public Property DebugPrint As Boolean = True

    Public Property LogWindowsEvents As Boolean = True

    Public Property LogWindowsName As String = "ErrorLog"

    Private moMenu As New System.Windows.Forms.ContextMenu
    Private WithEvents moMenuCopy As New System.Windows.Forms.MenuItem
    Private WithEvents moMenuView As New System.Windows.Forms.MenuItem

    Private bRestored As Boolean

    Public Sub LogMessage(iStatus As LogStates, sSummary As String, Optional sDetail As String = "", Optional sLocation As String = "",
                   Optional oEx As Exception = Nothing)
        ' Log message with elements provided separately
        Try
            Dim oDetail As New ErrorDetail(iStatus, sSummary, sDetail, sLocation, oEx)
            LogMessage(oDetail)
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Public Sub LogMessage(oDetail As ErrorDetail)
        ' Log a Message, taking into account repeats
        Try
            Dim bIgnoreRepeat As Boolean
            Dim sLocation As String

            If oLogItem.SubItems.Count < 3 Then                     ' First time - set up some dummy data
                oLogItem.SubItems.Add("")
                oLogItem.SubItems.Add("")
                oLogItem.SubItems.Add(0)
                oLogItem.SubItems.Add("")
            End If
            If (oLogItem.SubItems(1).Text = oDetail.Summary) And CollapseRepeatingMessages Then     ' It's a Repeat
                bIgnoreRepeat = True                                ' Don't write to Windows log
                oLogItem.SubItems(3).Text = Int(oLogItem.SubItems(3).Text) + 1
            Else                                                    ' It's a new item
                oLogItem = New ListViewItem
                oLogItem.StateImageIndex = oDetail.Status
                oLogItem.SubItems(0).Text = Strings.Format$(Now, "dd/MM HH:mm:ss")
                oLogItem.SubItems.Add(oDetail.Summary)
                oLogItem.SubItems.Add(oDetail.Detail)
                oLogItem.SubItems.Add("0")
                sLocation = oDetail.Location
                If Not oDetail.Ex Is Nothing Then
                    Dim st As New StackTrace(oDetail.Ex, True)
                    sLocation &= " L" & st.GetFrame(st.FrameCount - 1).GetFileLineNumber().ToString
                End If
                oLogItem.SubItems.Add(sLocation)
                'lstOutput.Items.Insert(0, oLogItem)
                AddItemToList(oLogItem)                             ' Handle invocation from different threads
            End If
            Debug.Print(oDetail.Summary)
            If LogWindowsEvents Then
                Dim iType As EventLogEntryType
                If oDetail.Status = LogStates.LSError Then
                    iType = EventLogEntryType.Error
                ElseIf oDetail.Status = LogStates.LSWarning Then
                    iType = EventLogEntryType.Warning
                Else
                    iType = EventLogEntryType.Information
                End If
                If Not bIgnoreRepeat Then
                    System.Diagnostics.EventLog.WriteEntry(LogWindowsName, oDetail.Summary & vbCrLf & oDetail.Detail, iType)
                End If
            End If
            'My.Application.Log.WriteEntry(oDetail.Summary & vbCrLf & oDetail.Detail)
            RaiseEvent ErrorLogged(oDetail)

        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Private Sub AddItemToList(oLogItem As ListViewItem)
        ' Add item to list handling invocation from different threads
        If lstOutput.InvokeRequired Then
            lstOutput.Invoke(New Action(Of Object)(AddressOf AddItemToList), oLogItem)
        Else
            lstOutput.Items.Insert(0, oLogItem)
        End If
    End Sub

    Public Sub LogStatus(sText As String)
        ' Just log to status bar and debug print
        Try
            Debug.Print(sText)
            lblStatus.Text = sText
            Application.DoEvents()
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Private Sub cmdHide_Click(sender As Object, e As EventArgs) Handles cmdHide.Click
        If ShowStatusBar Then
            lstOutput.Hide()
        Else
            Me.Hide()
        End If
    End Sub

    Private Sub ucErrorLog_SizeChanged(sender As Object, e As EventArgs) Handles Me.SizeChanged
        Try
            Dim iFixed As Integer = colTime.Width + colRpt.Width + colLocation.Width + 10
            Dim iFlexible As Integer = (Me.Width - iFixed) / 2
            If iFlexible > 0 Then
                colSummary.Width = iFlexible
                colDetail.Width = iFlexible
            End If

            cmdHide.Left = Me.Width - cmdHide.Width - 17
            cmdHide.Visible = ShowHideButton    ' Hide button if required
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Private Sub ucErrorLog_Load(sender As Object, e As EventArgs) Handles Me.Load
        Try
            lblStatus.Visible = ShowStatusBar

            ' Set up Copy Context Menu
            Me.moMenu.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {moMenuCopy, moMenuView})
            Me.moMenuCopy.Index = 0
            Me.moMenuCopy.Text = "&Copy"
            Me.moMenuView.Index = 1
            Me.moMenuView.Text = "&View"
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Private Sub lstOutput_MouseClick(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles lstOutput.MouseClick
        ' Right-click show context menu and set current filename for further actions
        Try
            If e.Button = MouseButtons.Right Then
                moMenu.Show(lstOutput, e.Location)
            End If
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Private Sub moMenuCopy_Click(sender As Object, e As EventArgs) Handles moMenuCopy.Click
        ' Copy selected item summary to clipboard
        Try
            Clipboard.SetText(TextSummary(lstOutput.SelectedItems(0)))
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Private Sub moMenuView_Click(sender As Object, e As EventArgs) Handles moMenuView.Click
        ' View selected item summary 
        Try
            MsgBox(TextSummary(lstOutput.SelectedItems(0)))
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Private Sub lstOutput_DoubleClick(sender As Object, e As EventArgs) Handles lstOutput.DoubleClick
        ' Double-click item to show summary
        Try
            MsgBox(TextSummary(lstOutput.SelectedItems(0)))
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Private Function TextSummary(oItem As ListViewItem) As String
        Try
            Dim sSummary As String = ""
            Select Case oItem.StateImageIndex
                Case 0
                    sSummary = "Info @ " & oItem.SubItems(0).Text & vbCrLf
                Case 1
                    sSummary = "Warning @ " & oItem.SubItems(0).Text & vbCrLf
                Case 2
                    sSummary = "Error @ " & oItem.SubItems(0).Text & vbCrLf
                Case 3
                    sSummary = "Clear @ " & oItem.SubItems(0).Text & vbCrLf
            End Select
            sSummary &= "Summary: " & oItem.SubItems(1).Text & vbCrLf &
                        "Detail: " & oItem.SubItems(2).Text & vbCrLf &
                        "Repeated: " & oItem.SubItems(3).Text & " times. Location: " & oItem.SubItems(4).Text
            Return sSummary
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Function

    Private Sub lstOutput_ColumnWidthChanged(sender As Object, e As ColumnWidthChangedEventArgs) Handles lstOutput.ColumnWidthChanged
        Try
            If bRestored Then SaveSetting(Application.ProductName, "Settings", Me.Name & "Widths", String.Join(",", Me.ColumnWidths))
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Public Sub RestoreColumnWidths()
        Try
            Dim sWidths As String = GetSetting(Application.ProductName, "Settings", Me.Name & "Widths", "")
            If sWidths <> "" Then
                Dim iWidths As New List(Of Integer)
                For Each sWidth In sWidths.Split(",")
                    iWidths.Add(CInt(sWidth))
                Next
                ColumnWidths = iWidths
            End If
            bRestored = True
        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Sub

    Public Property ColumnWidths As List(Of Integer)
        Get
            Dim oWidths As New List(Of Integer)
            For Each oColumn As ColumnHeader In lstOutput.Columns
                oWidths.Add(oColumn.Width)
            Next
            Return oWidths
        End Get
        Set(oWidths As List(Of Integer))
            Dim iCol As Integer = 0
            For Each iWidth In oWidths
                Dim oColumn As ColumnHeader = lstOutput.Columns(iCol)
                oColumn.Width = iWidth
                iCol += 1
            Next
        End Set
    End Property

    Public Function GetLogType(iState As LogStates) As String
        Try
            Select Case iState
                Case LogStates.LSInfo
                    Return "Information"
                Case LogStates.LSError
                    Return "Error"
                Case LogStates.LSWarning
                    Return "Warning"
                Case LogStates.LSClear
                    Return "Clear"
                Case Else
                    Return "Unknown Error Type " & iState
            End Select

        Catch ex As Exception
            Debug.Print(ex.Message)
        End Try
    End Function
End Class

Public Enum LogStates
    LSInfo = 0
    LSWarning = 1
    LSError = 2
    LSClear = 3
End Enum

Public Class ErrorDetail
    Public Property Status As LogStates
    Public Property Location As String
    Public Property Summary As String
    Public Property Detail As String
    Public Property Ex As Exception

    Public Sub New(iStatus As LogStates, sSummary As String, Optional sDetail As String = "", Optional sLocation As String = "",
                   Optional oEx As Exception = Nothing)
        Status = iStatus
        Summary = sSummary
        Detail = sDetail
        Location = sLocation
        Ex = oEx
    End Sub
End Class