﻿Imports System.Drawing
Imports System.Reflection.MethodBase
Imports System.Runtime.CompilerServices
Imports System.Windows.Forms
Imports ErrorLog
Imports ErrorLog.LogStates
Imports Microsoft.Office.Interop
Imports Microsoft.Office.Interop.Excel

Public Module modExcelVB
    ' Defines Extension Methods for rapid data access on Excel Worksheets

    Private sModule As String = "modExcelVB"
    Public Event ErrorRaised(oDetail As ErrorDetail)

    <Extension()>
    Public Function ExtractArray(oSheet As Worksheet, sTopLeft As String, Optional sRightColumn As String = "",
                                 Optional sExtentColumn As String = "") As CExcelArray
        ' Extract data starting at sTopLeft cell to right extent sRightColumn
        ' If sRightColumn blank then derive from length of first line
        ' Vertical extent defined by checking data below sExtentColumn
        Try
            Dim oData As Range
            Dim oArray As New CExcelArray
            If sExtentColumn = "" Then sExtentColumn = sTopLeft
            If sRightColumn = "" Then
                oData = oSheet.Range(oSheet.Range(sTopLeft), oSheet.Range(sTopLeft).End(XlDirection.xlToRight))
                oArray.aRows = oData.Value
                sRightColumn = GetExcelColumnName(oSheet.Range(sTopLeft).Column + oArray.aRows.Length - 1)
            End If
            oData = oSheet.Range(oSheet.Range(sExtentColumn), oSheet.Range(sExtentColumn).End(XlDirection.xlDown))
            oArray.aRows = oData.Value
            oArray.iRows = oArray.aRows.Length
            oData = oSheet.Range(oSheet.Range(sTopLeft), oSheet.Range(sRightColumn & oArray.iRows))
            oArray.aRows = oData.Value
            oArray.iCols = oArray.aRows.Length / oArray.iRows
            Return oArray
        Catch ex As Exception
            Throw New Exception(sModule & "." & GetCurrentMethod().Name & ":" & ex.Message, ex)
        End Try
    End Function

    <Extension()>
    Public Function WriteArray(oSheet As Worksheet, oRows As Array, iStartRow As Integer, iStartCol As Integer)
        Try
            Dim oRange As Range
            With oSheet
                oRange = .Range(.Cells(iStartRow, iStartCol),
                                .Cells(UBound(oRows, 1) - LBound(oRows, 1) + iStartRow,
                                       UBound(oRows, 2) - LBound(oRows, 2) + iStartCol))
            End With
            oRange.Value = oRows
        Catch ex As Exception
            Throw New Exception(sModule & "." & GetCurrentMethod().Name & ":" & ex.Message, ex)
        End Try
    End Function

    Public Function CreateRowArray(iCols As Integer) As String(,)
        ' Create 1-based array to hold write-back data
        Try
            Dim oRow(0, iCols) As String
            Return oRow
        Catch ex As Exception
            Throw New Exception(sModule & "." & GetCurrentMethod().Name & ":" & ex.Message, ex)
        End Try
    End Function


    <Extension()>
    Public Function ClearRowArray(oRow As Array)
        ' Empty array to hold write-back data
        Try
            For i = 1 To UBound(oRow, 2)
                oRow(1, i) = ""
            Next
        Catch ex As Exception
            Throw New Exception(sModule & "." & GetCurrentMethod().Name & ":" & ex.Message, ex)
        End Try
    End Function


    <Extension()>
    Public Function ExtractStringArray(oArray As System.Array, iDimension As Integer, Optional iPadLeft As Integer = 0) As String()
        ' Extract a single string array from a multi-dimensional string array
        ' Preserves the upper bound so Excel code works, pushes empty strings into positions below lower bound
        ' Optionally pads with a specified number of columns to the left, so we can preserve column numbering
        Try
            Dim iLBound As Integer = oArray.GetLowerBound(iDimension)
            Dim iUBound As Integer = oArray.GetUpperBound(iDimension)
            Dim sArray(iUBound + iPadLeft) As String
            For i = 0 To iUBound + iPadLeft
                sArray(i) = ""
            Next
            For i = iLBound To iUBound
                If Not oArray(iDimension, i) Is Nothing Then sArray(i + iPadLeft) = oArray(iDimension, i)
            Next
            Return sArray

        Catch ex As Exception
            Throw New Exception(sModule & "." & GetCurrentMethod().Name & ":" & ex.Message, ex)
        End Try
    End Function

    <Extension()>
    Public Function AddCommentWithStyle(oCell As Object, sComment As String, sStyle As String)
        Try
            oCell.Style = sStyle
            oCell.AddComment(sComment)
        Catch ex As Exception
            Throw New Exception(sModule & "." & GetCurrentMethod().Name & ":" & ex.Message, ex)
        End Try
    End Function

    Public Function ValidateCellFormat(oCell As Excel.Range, sFormat As String) As Boolean
        Try
            oCell.NumberFormat = sFormat
            Return True
        Catch ex As Exception       ' Any exception means invalid format
            Return False
        End Try
    End Function

    Public Function GetExcelColumnName(columnNumber As Integer) As String
        Try
            Dim dividend As Integer = columnNumber
            Dim columnName As String = String.Empty
            Dim modulo As Integer

            While dividend > 0
                modulo = (dividend - 1) Mod 26
                columnName = Convert.ToChar(65 + modulo).ToString() & columnName
                dividend = CInt((dividend - modulo) / 26)
            End While

            Return columnName
        Catch ex As Exception
            Throw New Exception(sModule & "." & GetCurrentMethod().Name & ":" & ex.Message, ex)
        End Try
    End Function

    Public Class CExcelArray
        Public Property aRows As System.Array
        Public Property iRows As Integer
        Public Property iCols As Integer
    End Class
End Module

